function [pos test] = LEEDSP_data
% this function is very dataset specific, you need to modify the code if
% you want to apply the pose algorithm on some other dataset

% it converts the various data format of different dataset into unique
% format for pose detection 
% the unique format for pose detection contains below data structure
%   pos:
%     pos(i).im: filename for the image containing i-th human 
%     pos(i).point: pose keypoints for the i-th human
%   test:
%     test(i).im: filename for i-th testing image
% This function also prepares flipped images for training.

load data/LEEDSP/joints.mat
% load LEEDSP/joints_relabel.mat;
posims = 'data/LEEDSP/im%.4d.jpg';

trainfrs = 1:1000; % training frames
testfrs = 1001:2000; % testing frames

% -------------------
% grab positive annotation and image information
pos = [];
numpos = 0;
for fr = trainfrs
  numpos = numpos + 1;
  pos(numpos).im = sprintf(posims,fr);
  pos(numpos).point = joints(1:2,:,fr)';
  pos(numpos).invis = joints(3,:,fr);
end

% -------------------
% flip positive training images
if ~exist('data/LEEDSP/imflip', 'dir')
  mkdir('data/LEEDSP/imflip');
end
posflipims = ['data/LEEDSP/imflip/im%.4d.jpg'];
for n = 1:length(pos)
  if exist(sprintf(posflipims,n),'file')
    continue;
  end
  im = imread(pos(n).im);
  im_flip = im(:,end:-1:1,:);
  imwrite(im_flip,sprintf(posflipims,n));
end

% -------------------
% flip labels for the flipped positive training images
% mirror property for the keypoint, please check your annotation for your
% own dataset
mirror = [6 5 4 3 2 1 12 11 10 9 8 7 13 14];
for n = 1:length(pos)
  im = imread(pos(n).im);
  width = size(im,2);
  numpos = numpos + 1;
  pos(numpos).im = sprintf(posflipims,n);
  for p = 1:size(pos(n).point,1)
    pos(numpos).point(mirror(p),1) = width - pos(n).point(p,1) + 1;
    pos(numpos).point(mirror(p),2) = pos(n).point(p,2);
    pos(numpos).invis(mirror(p)) = pos(n).invis(p);
  end
end


% -------------------
% grab testing image information
test = [];
numtest = 0;
for fr = testfrs
  numtest = numtest + 1;
  test(numtest).im = sprintf(posims,fr);
  test(numtest).point = joints(1:2,:,fr)';
end
