
global cachedir opts strparam

opts.expdir    = 'exprst';
opts.train_iter = 2;
% change the maxmem setting to fix specific device (default 6GB memory)
opts.maxmem    = 6;

sbin = 4;
name = 'LEEDSP';
suffix = 'struct24';

cachedir = sprintf('%s/cache-%s-sbin%d-%s/', opts.expdir, name, sbin, suffix);
if ~exist(cachedir, 'dir')
  mkdir(cachedir);
end

neg = load_data('INRIA');
[pos14p test] = load_data(name);
[scale, boxsize] = pointreg(pos14p, sbin);

% build up Latent Tree / update strparam & pos
fprintf('Generating latent tree structure ... \n');
strparam = getCombStructure24p();
pos_lt = getPoselet(pos14p, strparam);
adjG = getLatentTree(pos_lt, scale, [], 0);
[~, ~, pred] = treeBFS(adjG, 1);
% pred = [0  14  19  18  20  21  2  12  2  15  8  7  2  1  9  10  4  3  1  23  5  6  1  23];

strparam = getCombStructure24p(pred, 1);
pos = getPoselet(pos14p, strparam);
pos = pointtobox(pos, strparam, boxsize);

% train visual clusters
fprintf('\nTrain visual clusters (%d parts) ... \n', strparam.numparts);
pos = trainvisualcluster(name,pos,scale,sbin,suffix);
fprintf('Visual clusters (K) : ');
for pi=1:length(strparam.K), fprintf('%d ',strparam.K(pi)); end
fprintf('\n');

% train the model
fprintf('\nStart model training ... \n[%s] \n', cachedir);
model = trainmodel(name,pos,neg,strparam,sbin,suffix);

% testing
model.thresh   = min(model.thresh,-2);
[boxes points] = testmodel(name,model,test,suffix);

% evaluation
fprintf('\nEvaluation (test %d images) ... \n', length(test));
[detRate PCP R] = PARSE_eval_pcp('PARSE',points,test,strparam);
fprintf('detRate=%.3f, PCP=%.3f, detRate*PCP=%.3f\n',detRate,PCP,detRate*PCP);

partstr = {'torso', 'head', 'uleg', 'lleg', 'uarm', 'larm'};
I = [1  2   3  3  4  4  5  5  6  6 ];
J = [1  10  2  3  4  5  6  7  8  9 ];
S = [1  1   0.5*ones(1,8)];
A = full(sparse(I,J,S,6,10));

rst = R*detRate;
rst = A*rst(:);
for pi = 1:size(rst,1)
  fprintf('  %s=%.1f%%', partstr{pi}, rst(pi)*100);
end
fprintf('\n');
save([cachedir name '_pcp_' suffix],'detRate','PCP','R');

showResults(test, name, model, strparam, suffix);

