function [boxes,model,loss,ex] = detect_warp(im, model, thresh, bbox, overlap, id, label, loss)

if nargin > 3 && ~isempty(bbox) && label == 1
  latent = true;
  thresh = -1e10;
else
  latent = false;
end

% Define global QP if we are writing features
% Randomize order to increase effectiveness of model updating
write = false;
if nargin < 4
  bbox = [];
  overlap = 0.6;
end

if nargin > 5,
  global qp;
  write  = true;
end
if nargin < 6
  id = 0;
end
if nargin < 7
  label = 0;
end
if nargin < 8
  loss = 0;
end

ex = [];  
[boxes, exvec, pyra] = detector(im, model, thresh, write, bbox, overlap);

levels   = 1:length(pyra.feat);
if write && ~latent
  levels = levels(randperm(length(levels)));
end

if isempty(boxes)
  return;
end

levelvec = cell2mat({exvec.id}');
levelvec = levelvec(:,3);

if write && ~latent,
  w = -(qp.w + qp.w0.*qp.wreg) / qp.Cneg;
end

for rlevel = levels
  levelind = find(levelvec == rlevel);
  
  for i = 1:length(levelind)
    id = levelind(i);
    
    featmap = pyra.feat{rlevel};
    
    ex = exvec(id);
    ex.id(1) = label;
    ex.id(2) = id;
    rscore = boxes(id, end);
    
    if write && ~latent,
      qp_write(ex, featmap);
      loss  = loss + qp.Cneg*max(1+rscore,0);
    end
  end
  
  % Crucial DEBUG assertion:
  % If we're computing features, assert extracted feature re-produces score
  % (see qp_writ.m for computing original score)
  if write && ~latent && ~isempty(levelind) && qp.n < length(qp.a),
     assert(abs((score(w,qp.x,qp.n) - rscore)) < 1e-5);
  end
  
  % Optimize qp with coordinate descent, and update model
  if write && ~latent && ...
      (qp.obj < 0 || loss/qp.obj > .01 || qp.n == length(qp.sv))
    model = optimize(model);
    loss = 0;
  end
end

if latent && ~isempty(boxes),
  boxes = boxes(end,:);
  if write,
    qp_write(ex, featmap);
  end
end

% Update QP with coordinate descent
% and return the asociated model
function model = optimize(model)
  global qp
  fprintf('.');  
  if qp.obj < 0 || qp.n == length(qp.a),
    qp_opt(1);
    qp_prune();
  else
    qp_one();
  end
  model = vec2model(qp_w,model);
 
