function strparam = getCombStructure24p(pred, post_processing)

if nargin < 2
  post_processing = 0;
  pred = [];
end

% fprintf('Structure setting: 24 combined + primitive parts.\n');
strparam.combs = {[13 14 9 10 3 4], [2 3 3 4 4 5], ...
                  [9 8], [8 7], [10 11], [11 12], [3 2], [2 1], [4 5], [5 6]};

strparam.prims = [1:14];
strparam.ind14p = [11:24];
strparam.level = 2;
strparam.pa = [0 1   1 4 1 6 2 8 2 10   9 9 8 10 11 11 5 5 4 6 7 7 3 3];
strparam.K  = [8 10 16*ones(1,8) 8*ones(1,14)];
numparts = length(strparam.K);
strparam.numparts = numparts;

if isfield(strparam, 'prims')
  
  nc = length(strparam.combs);
  for i = 1:nc
    poselet = strparam.combs{i};
    strparam.combs{i} = poselet + nc;
  end
  strparam.combs = [strparam.combs, num2cell(strparam.prims+nc)];
  strparam = rmfield(strparam, 'prims');
  
end

strparam.colorset = {'y', ...
  [ 1.0000    0.500    0], ...
  'c', 'm', 'c', 'm', 'g', 'b', 'g', 'b', ...
  'b', 'g', 'g', 'g', 'g', 'b', 'm', 'c', 'c', 'c', 'c', 'm', 'r', 'r'};

strparam.order = [1:numparts];

%% reordering according to tree structure
if (post_processing)
  adjG = spconvert([ [2:numparts]' pred(2:end)' ones(numparts-1, 1); ...
    [numparts numparts 0] ]);
  adjG = adjG + adjG';
  [depth, map, pred] = treeBFS(adjG, 1);
  
  strparam.map = map;
  
  % re-arrange structure
  old_order = [1:numparts];
  bfs_order = old_order;
  bfs_order(map(old_order)) = old_order;
  
  ind14p = strparam.ind14p;
  ind14p = map(ind14p);
  
  new_pred = [0, map(pred(2:end))];
  new_pred = new_pred(bfs_order);
  
  strparam.pa = new_pred;
  strparam.order = bfs_order;
  strparam.ind14p = ind14p;
  
  order = strparam.order;
  strparam.combs = strparam.combs(order);
  for p = 1:strparam.numparts
    strparam.combs{p} = map(strparam.combs{p});
  end
  strparam.K = strparam.K(order);
  strparam.colorset = strparam.colorset(order);
end


