function adjG = getLatentTree(pos, scale, root, display)

if (nargin < 3 || isempty(root))
  root = 1;
end

if (nargin < 4)
  display = 0;
end

useDistances = 1;

N = length(pos);
Dim = size(pos(1).point, 1);

% collect data samples
ptsAll = zeros(Dim, 2, N);
ptsCtr = zeros(N,2);
for i = 1:N
  ptsAll(:,:,i) = pos(i).point ./ scale(i);
end
for i = 1:N
  ptsCtr(i,:) = mean(ptsAll(:,:,i), 1);
end
for i = 1:N
  ptsAll(:,:,i)=bsxfun(@minus, ptsAll(:,:,i), ptsCtr(i, :));
end

ptsMin = floor(min(min(ptsAll, [], 3), [], 1));
ptsMax = floor(max(max(ptsAll, [], 3), [], 1));
ptsBound = max(-ptsMin, ptsMax) + [5,5];
ptsAll = bsxfun(@plus, ptsAll, ptsBound);

% create cov mat
rng(1234);
numSamples = N;
randind = randi(N, [1,numSamples]);
meanpts = mean(ptsAll, 3);
ptsOffset = bsxfun(@minus, ptsAll, meanpts);

cov_mat  = zeros(Dim, Dim);
dist_mat = zeros(Dim, Dim);
for i = 1:Dim
  for j = 1:Dim
    tmp = squeeze(ptsAll(i,:,randind).*ptsAll(j,:,randind));
    cov_mat(i,j) = sum(tmp(:))/(numSamples-1);
    
    tmp = squeeze(ptsOffset(i,:,randind).*ptsOffset(j,:,randind));
    dist = sqrt(sum( (meanpts(i,1)-meanpts(j,1)).^2 ));
    dist_mat(i,j) = sum(tmp(:))/(numSamples-1) + dist;
  end
end
sample_cov = cov_mat;

m = size(sample_cov,1);
D = diag(1./sqrt(diag(sample_cov)));
rho_mat = D*sample_cov*D;
distance = -log(abs(rho_mat));

if (any(any(isnan(distance))))
  keyboard;
end

tic;
[adjG,edgeD] = CLRG(distance, useDistances, numSamples);
toc;

if (display)

  figure; clf;
  drawLatentTree(adjG,m,root);
  
  % show sample image
  id = 20;
  figure(12);
  plot(ptsAll(:,1,id), ptsAll(:,2,id), 'bx', 0, 0, 'r+'); axis image; hold on
  set(gca, 'ydir', 'reverse')
  xlim([0, 2*ptsBound(1)]);
  ylim([0, 2*ptsBound(2)]);
  text(ptsAll(:,1,id), ptsAll(:,2,id), num2str([1:Dim]'), 'VerticalAlignment','bottom');
  
end

