function [pos] = getPoselet(pos14p, strparam)
% function [pos] = getPoselet(pos14p, strparam)
% Input: 
%   pos(i).point: contains 14x2 joints coordinates, denotes:
%     1:left ankle, 2:left knee, 3:left hip,
%     4:right hip, 5:right knee, 6:right ankle,
%     7:left wrist, 8:left elbow, 9:left shoulder,
%     10:right shoulder, 11:right elbow, 12:right wrist,
%     13:head bottom, 14:head top.
%   strparam: indicate structure of labeled points
%     strparam.numparts: total number of parts (combined+primitive)
%     strparam.combs: combined parts
%     strparam.ind14p: index of the original 14 points
%
% Output:
%   pos:
%     pos(i).im: filename for the image containing i-th human 
%     pos(i).point: center points of all part
%     pos(i).poselet: contents of i-th poselet

pos = pos14p;
npos  = length(pos14p);
nt = strparam.numparts;

% save original labels
for i = 1:npos
  pos(i).point14 = pos14p(i).point;
  pos(i).point = zeros(nt, 2);
  pos(i).point(strparam.ind14p, :) = pos14p(i).point;
end

% build center transform matrix
I = [];
J = [];
S = [];
for i = 1:nt
  clen = length(strparam.combs{i});
  I = [I ones(1, clen)*i];
  J = [J strparam.combs{i}];
  S = [S ones(1, clen)./clen];
end
A = full(sparse(I,J,S,nt,size(pos(1).point,1)));

for i = 1:npos
  pos(i).point = A * pos(i).point; 
  pos(i).poselet = cell(1, nt);
  for p = 1:nt
    if (length(strparam.combs{p}) > 1)
      % poselet : center_point, point1, point2, ...
      plen = 1+length(strparam.combs{p});
      poselet = pos(i).point(p, :);
      for j = 2:plen
        poselet(j,:) = pos(i).point(strparam.combs{p}(j-1),:);
      end
      pos(i).poselet{p} = reshape(poselet', 1, plen*2);
    else
      % primitive part
      pos(i).poselet{p} = pos(i).point(p, :);
    end
  end
  
end







