function jointmodel = buildmodel(name,model,pa,def,idx,idx_map,K)
% jointmodel = buildmodel(name,model,pa,def,idx,K)
% This function merges together separate part models into a tree structure

global cachedir strparam

jointmodel.bias    = struct('w',{},'i',{});
jointmodel.defs    = struct('w',{},'i',{},'anchor',{});
jointmodel.filters = struct('w',{},'i',{});
jointmodel.components{1} = struct('ctnum',{}, 'ctmap', {}, 'ptnum', {}, 'biasid',{},'defid',{}, ...
                                  'vtmap',{}, 'filterid',{},'parent',{});

jointmodel.maxsize  = model.maxsize;
jointmodel.minsize  = model.minsize;
jointmodel.level = strparam.level;
jointmodel.maxdefsize = [1 1];
jointmodel.interval = model.interval;
jointmodel.sbin = model.sbin;
jointmodel.len = 0;

% add children
for i = 1:length(pa)
  child = i;
  parent = pa(child);
  assert(parent < child);
  
  cls = [name '_part_' num2str(child) '_mix_' num2str(K(i))];
  load([cachedir cls]);
  
  p.ctnum = max(idx{child});    % child node type number (visual x geometry)
  p.ctmap = idx_map{child};
  if parent > 0
    p.ptnum = max(idx{parent}); % parent node type number (visual x geometry)
  else
    p.ptnum = 0;
  end
  
  % add bias
  p.biasid = [];
  if parent == 0
    nb  = length(jointmodel.bias);
    b.w = 0;
    b.i = jointmodel.len + 1;
    jointmodel.bias(nb+1) = b;
    jointmodel.len = jointmodel.len + numel(b.w);
    p.biasid = nb+1;
  else
    for k = 1:p.ctnum
      for l = 1:p.ptnum
        nb = length(jointmodel.bias);
        b.w = 0;
        b.i = jointmodel.len + 1;
        jointmodel.bias(nb+1) = b;
        jointmodel.len = jointmodel.len + numel(b.w);
        p.biasid(l,k) = nb+1;
      end
    end
  end

  vtmap = true(p.ptnum, p.ctnum);  % valid type map
  
  % add deformation parameter
  p.defid = [];
  if parent > 0
    for k = 1:p.ctnum
      for l = 1:p.ptnum
        nd  = length(jointmodel.defs);
        d.w = [0.01 0 0.01 0];
        d.i = jointmodel.len + 1;
        pidx = find(idx{parent}==l);
        cidx = find(idx{child}==k);
        subidx = intersect(pidx, cidx);
        if isempty(subidx)
          x = Inf;
          y = Inf;
          vtmap(l,k) = false;
        else
          x = mean(def{child}(subidx,1) - def{parent}(subidx,1));
          y = mean(def{child}(subidx,2) - def{parent}(subidx,2));
        end
        d.anchor = round([x+1 y+1 0]);
        defsize = d.anchor([2,1]);
        defsize = max(defsize, 1-defsize)+1;
        if (x < Inf && y < Inf)
          jointmodel.maxdefsize = max(jointmodel.maxdefsize, defsize);
        end
        jointmodel.defs(nd+1) = d;
        jointmodel.len = jointmodel.len + numel(d.w);
        p.defid(l,k) = [nd+1];
        
      end
    end
  end
  
  p.vtmap = vtmap;

  % add filter
  p.filterid = [];
  for k = 1:p.ctnum  
    nf  = length(jointmodel.filters);
    f.w = model.filters(k).w;
    f.i = jointmodel.len + 1;
    jointmodel.filters(nf+1) = f;
    jointmodel.len = jointmodel.len + numel(f.w);
    p.filterid = [p.filterid nf+1];
  end

  p.parent = parent;
  np = length(jointmodel.components{1});
  jointmodel.components{1}(np+1) = p;
  jointmodel.maxsize = max(jointmodel.maxsize, model.maxsize);
  jointmodel.minsize = min(jointmodel.minsize, model.minsize);
end