function [raidx, raidxset, boxes] = clusterRatioAspect(boxes, scale, K, kmean_iter)
% cluster boxes into K groups, with respect to ratio aspects 
%   boxes : input sample boxes with n rows, each contains [x1, y1, x2, y2]
%   scale : relative scales within dataset
%   K     : cluster number
%   kmean_iter : kmean repeat times
%   
%   raidx : cluster labels for each sample
%   raidxset : labels set [1..K]
%   boxes    : output boxes with n rows [x1, y1, x2, y2]
%

% initial clustering in terms of poselet size
boxsize = [boxes(:,4)-boxes(:,2), boxes(:,3)-boxes(:,1)]./[scale', scale'];
boxsize2 = zeros(size(boxsize));
ravec= boxsize(:,1)./boxsize(:,2);

[raidx, rasets] = kmeans_wrap(ravec, K, kmean_iter);
raidxset = unique(raidx);
K = length(rasets);

boxsets = zeros(K, 2);

for i = 1:K
  subidx = find(raidx==i);
  boxsets(i,2) = median(boxsize(subidx,2));
  boxsets(i,1) = boxsets(i,2)*rasets(i);
  
  boxsize2(subidx, :) = repmat(boxsets(i,:), length(subidx), 1);
end

boxsize2 = boxsize2./2.*[scale', scale'];
ctrs = [boxes(:,4)+boxes(:,2), boxes(:,3)+boxes(:,1)]./2;
boxes(:,[2,1]) = ctrs-boxsize2;
boxes(:,[4,3]) = ctrs+boxsize2;

