#include <math.h>
#include <stdint.h>
#include "mex.h"
#include "matrix.h"
#include "memory.h"

void mexFunction( int nlhs, mxArray *plhs[],
                  int nrhs, const mxArray *prhs[] )
{

  if (mxIsStruct(prhs[0]) == false) mexErrMsgTxt("Arguement 1 is not struct");
  if (mxIsDouble(prhs[1]) == false) mexErrMsgTxt("Arguement 2 is not double");
  if (mxIsDouble(prhs[2]) == false) mexErrMsgTxt("Arguement 3 is not double");
  if (mxIsDouble(prhs[3]) == false) mexErrMsgTxt("Arguement 4 is not double");
  if (mxIsDouble(prhs[4]) == false) mexErrMsgTxt("Arguement 5 is not double");
  if (mxIsDouble(prhs[5]) == false) mexErrMsgTxt("Arguement 6 is not double");
  if (mxIsDouble(prhs[6]) == false) mexErrMsgTxt("Arguement 7 is not double");

	int nBlocks = mxGetNumberOfElements(prhs[0]);
	const mxArray *pBlock = (mxArray *) prhs[0];
	const double C = mxGetScalar(prhs[1]);
	const double *pW0 = (double *) mxGetPr(prhs[2]);
	const double *pWreg = (double *) mxGetPr(prhs[3]);
	const int xlen = (int) *((double *) mxGetPr(prhs[4]));
	const int negfac = (int) *((double *) mxGetPr(prhs[5]));

	mxArray *mxX = mxCreateDoubleMatrix(xlen, 1, mxREAL);
	double *X = (double *) mxGetPr(mxX);
	memset(X, 0, sizeof(double) * mxGetNumberOfElements(mxX));
	double norm = 0;
	double bias = 1;

	mxArray *mxBlockX = NULL;
	mxArray *mxFeatPatch = NULL;
	int i1, blen, is, index;
	int j = 0;

	j = 0;
	X[j++] = nBlocks;

	for (index = 0; index < nBlocks; index++) {
		i1 = mxGetScalar(mxGetField(pBlock, index, "i"));
		mxBlockX = (mxArray *) mxGetField(pBlock, index, "x");
		double *px = (double *) mxGetPr(mxBlockX);
		blen = mxGetNumberOfElements(mxBlockX);
		double *pWork = NULL;

		// if block.x is index of featmap, we need to get real x from featmap
		if (blen == 5) {

			int m = (px[4] - px[3] + 1);
			int n = (px[2] - px[1] + 1);
			int dim = px[0];

			const mxArray *pWorkmap = (mxArray *) prhs[6];
			double *featmap = (double *) mxGetPr(pWorkmap);
			const int *dims = mxGetDimensions(pWorkmap);

			double *pMN = NULL;
			double *pN = NULL;
			double *pM = NULL;
			blen = dim * m * n;
			pWork = (double *) malloc(sizeof(double) * (blen * 2));

			int pi = 0;
			for (int di = 0; di < dims[2]; di++) {
				pMN = featmap + di * dims[0] * dims[1];
				for (int xi = px[1] - 1; xi < px[2]; xi++) {
					pN = pMN + xi * dims[0] + int(px[3]) - 1;
					for (int yi = px[3] - 1; yi < px[4]; yi++) {
						pWork[pi++] = *(pN++);
					}
				}
			}

		} else {
			pWork = (double *) malloc(sizeof(double) * blen);
			for (is = 0; is < blen; is++) {
				pWork[is] = px[is];
			}
		}

		double sum = 0;
		for (is = 0; is < blen; is++) {
			pWork[is] = (pWork[is]) * negfac;
			sum += pW0[is + i1 - 1] * pWork[is];
		}
		bias = bias - sum;

		sum = 0;
		for (is = 0; is < blen; is++) {
			pWork[is] = pWork[is] * C / pWreg[is + i1 - 1];
			sum += pWork[is] * pWork[is];
		}
		norm = norm + sum;

		X[j++] = i1;
		X[j++] = i1 + blen - 1;
		for (is = 0; is < blen;) {
			X[j++] = pWork[is++];
		}

		if (pWork != NULL) {
			free(pWork);
			pWork = NULL;
		}
	}

	plhs[0] = mxX;
	plhs[1] = mxCreateDoubleMatrix(1, 1, mxREAL);
	double *p = (double *) mxGetPr(plhs[1]);
	*p = norm;
	plhs[2] = mxCreateDoubleMatrix(1, 1, mxREAL);
	p = (double *) mxGetPr(plhs[2]);
	*p = bias;
	return;
}

/*
%% [x, norm, bias] = ex2vec(ex.blocks, C, w0, wreg, xlen, negfac, featmap)
*/
