function model = initmodel(pos,p,sbin,scale,tsize)

% all the part templates currently have the same size, so no need to
% compute template size for every part
w = zeros(1,length(pos));
h = zeros(1,length(pos));
for n = 1:length(pos)
  w(n) = (pos(n).x2(p) - pos(n).x1(p) + 1)/scale(n);
  h(n) = (pos(n).y2(p) - pos(n).y1(p) + 1)/scale(n);
end
xx = -2:.02:2;
filter = exp(-[-100:100].^2/400);
aspects = hist(log(h./w), xx);
aspects = convn(aspects, filter, 'same');
[peak, I] = max(aspects);
aspect = exp(xx(I));

% pick 20 percentile area
areas = sort(h.*w);
if (length(areas)>=5)
  area = areas(floor(length(areas) * 0.6));
else
  area = areas(ceil(length(areas)/2));
end

% pick dimensions
nw = sqrt(area/aspect);
nh = nw*aspect;
nf = length(features(zeros([3 3 3]),1));

% size of HOG features
if nargin < 2
  model.sbin = 8;
else
  model.sbin = sbin;
end

% pick dimensions
if nargin < 5
  tsize = floor([floor(nh/model.sbin) floor(nw/model.sbin)]/2)*2+1;
  tsize = [tsize nf];
else 
  tsize = [tsize nf];
end

% bias
b.w = 0;
b.i = 1;

% filter
f.w = zeros(tsize);
f.i = 1+1;

% set up one component model
c(1) = struct('ctnum',1, 'ctmap', 1, 'ptnum', 0, ...
  'biasid',1,'defid',[], 'vtmap', [], 'filterid',1,'filterfixid',[],'fwid',[],...
  'parent',0);
model.bias(1)    = b;
model.defs       = [];
model.filters(1) = f;
model.fws        = [];
model.components{1} = c;

% initialize the rest of the model structure
model.interval = 5;
model.maxsize = tsize(1:2);
model.minsize = tsize(1:2);
model.level = 1;
model.maxdefsize = [1 1];
model.len = 1+prod(tsize);

