#include "mex.h"
#include <math.h>
#include <string.h>


void sparse2dense(const float *pX, int M, double *pW, int wlen, double A)
{
	float *pWork = NULL;
	double *pWork2 = pW;

	int bnum = (int)(pX[0]);
	int j=0, i1, i2;
	for (int i=0; i<bnum; i++) {
		i1 = (int)(pX[j+1])-1;
		i2 = (int)(pX[j+2])-1;
		pWork = (float *)(pX+j+3);
		for (int k=i1; k<=i2; ++k) {
			pWork2[k] += (*(pWork++))*A;
		}
		j += 3+i2-i1;
	}
}


// matlab entry point
// qp_prune_cc(qp, emidx, svidx, n_new)
void mexFunction(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[]) { 
	if (nrhs != 4)
		mexErrMsgTxt("Wrong number of inputs");
	if (nlhs != 0)
		mexErrMsgTxt("Wrong number of outputs");

	if (mxIsStruct(prhs[0]) == false)
		mexErrMsgTxt("Arguement 1 is not struct");

	mxArray *mxQp = (mxArray *)prhs[0];

	mxArray *mxSv = mxGetField(mxQp, 0, "sv");
	bool *pSv = (bool *)mxGetData(mxSv);
	mxArray *mxA = mxGetField(mxQp, 0, "a");
	double *pA = (double *)mxGetData(mxA);
	mxArray *mxD = mxGetField(mxQp, 0, "d");
	double *pD = (double *)mxGetData(mxD);
	mxArray *mxI = mxGetField(mxQp, 0, "i");
	int *pI = (int *)mxGetData(mxI);
	mxArray *mxB = mxGetField(mxQp, 0, "b");
	float *pB = (float *)mxGetData(mxB);
	mxArray *mxX = mxGetField(mxQp, 0, "x");
	float *pX = (float *)mxGetData(mxX);
	mxArray *mxL = mxGetField(mxQp, 0, "l");
	double *pL = (double *)mxGetPr(mxL);
	mxArray *mxW = mxGetField(mxQp, 0, "w");
	double *pW = (double *)mxGetPr(mxW);

	double *pN = (double *)mxGetPr(mxGetField(mxQp, 0, "n"));
	int N = (int)(*pN);
	int M = (int)mxGetM(mxX);

	mxArray *emidx = (mxArray *)prhs[1];  // index of empty entries in [1:qp.n]
	mxArray *svidx = (mxArray *)prhs[2];  // inverse index of sv entries
	double *pEmidx = (double *)mxGetPr(emidx);
	double *pSvidx = (double *)mxGetPr(svidx);
	int idxlen = mxGetNumberOfElements(emidx);

	int N_new = int(*(double *)mxGetPr((mxArray*)prhs[3]));

	*pN = N_new;
	N = N_new;

	int i, j, k, t;

	// do prune
	for (i=0; i < idxlen; ++i) {
		j = (int)(pEmidx[i])-1;
		k = (int)(pSvidx[i])-1;

		pB[j] = pB[k];
		pD[j] = pD[k];
		pA[j] = pA[k];
		pSv[j] = pSv[k];

		// copy qp.x
		float *pXSrc = pX+M*k;
		float *pXDst = pX+M*j;
		for (t=0; t < M; ++t) {
			*(pXDst++) = *(pXSrc++);
		}

		// copy qp.i
		int dim = (int)mxGetM(mxI);
		int *pXSrc2 = pI+dim*k;
		int *pXDst2 = pI+dim*j;
		for (t=0; t < dim; ++t) {
			*(pXDst2++) = *(pXSrc2++);
		}

	}

	// calculate l and w
	*pL = 0;
	int wlen = mxGetNumberOfElements(mxW);
	memset(pW, 0, sizeof(double)*wlen);

	for (i=0; i < N_new; ++i) {
		*pL += (double(pB[i]))*(pA[i]);
		sparse2dense(pX+M*i, M, pW, wlen, pA[i]);
	}
}


