function qp_write(ex, featmap)
% qp_write(ex)
% where ex.id   = K X 1
%       ex.blocks(j).i = starting index of j^th feature block
%       ex.blocks(j).x = feature block
%       final feature  = [ex.blocks(:).x]
%
% Write ex to a QP of the form:
% min_{w,e}  ||(w-w0)*r||^2 + sum_i c_i e_i
%      s.t.  w x_ij >= 1 - e_i 
%
% We can write the above QP in "standard" form
% with the following substitution: v = (w-w0)*r
% min_{v,e}  ||v||^2 + sum_i e_i
% s.t. v x'_ij >= b'_ij - e_i   
%
% where  x'_ij = c_i*(x_ij/r)
%        b'_ij = c_i*(1 - w0*x_ij)

  global qp
  
  if qp.n == length(qp.a),
    return;
  end
  
  label = ex.id(1) > 0;

  if label,
    C = qp.Cpos;
    negfac = 1;
  else
    C = qp.Cneg;
    negfac = -1;
  end
 
  % Ensure there are no duplicate blocks
  is = sort([ex.blocks.i]);
  assert(~any(is(2:end) == is(1:end-1)));
   
  qp.n = qp.n + 1;
  i    = qp.n;
  
  [x, norm, bias] = ex2vec(ex.blocks, C, qp.w0, qp.wreg, size(qp.x,1), negfac, featmap);
  
  qp.x(:,i) = x;
  qp.d(i)   = norm;
  qp.b(i)   = C*bias;
  qp.i(:,i) = ex.id;
  qp.sv(i)  = 1;
 
