% Computes expected number of nonzeros in sparse feature vector 
function len = sparselen(model)

  len       = 0;
  numblocks = 0;
  for c = 1:length(model.components),
    feat = zeros(model.len,1);
    for p = model.components{c},
      if ~isempty(p.biasid)
        x = model.bias(p.biasid(1));
        i1 = x.i;
        i2 = i1 + numel(x.w) - 1;
        feat(i1:i2) = 1;
        numblocks = numblocks + 1;
      end
      if ~isempty(p.defid)
        x  = model.defs(p.defid(1));
        i1 = x.i;
        i2 = i1 + numel(x.w) - 1;
        feat(i1:i2) = 1;
        numblocks = numblocks + 1;
      end
      if ~isempty(p.filterid)
        % find the max filter size
        x  = model.filters(p.filterid(1));
        for fi = p.filterid
          if numel(x.w)<numel(model.filters(fi).w)
            x = model.filters(fi);
          end
        end

        i1 = x.i;
        i2 = i1 + numel(x.w) - 1;
        feat(i1:i2) = 1;
        numblocks = numblocks + 1;
      end
      
      if isfield(p, 'fwid') && ~isempty(p.fwid)
        x  = model.fws(p.fwid(1));
        i1 = x.i;
        i2 = i1 + numel(x.w) - 1;
        feat(i1:i2) = 1;
        numblocks = numblocks + 1;
      end
    end
    
    % Number of entries needed to encode a block-sparse representation
    %   1 + numberofblocks*2 + #nonzeronumbers
    n = 1 + numblocks*2 + sum(feat);
    len = max(len,n);
  end
