function [scale, boxsize] = pointreg(pos, sbin)
% normalize data using Yang and Ramanan's part setting

I = [1  2  3  4   4   5  6   6   7  8   8   9   9   10 11  11  12 13  13  14 ...
  15 16  16  17 18  18  19 20  20  21  21  22 23  23  24 25  25  26];
J = [14 13 9  9   8   8  8   7   7  9   3   9   3   3  3   2   2  2   1   1 ...
  10 10  11  11 11  12  12 10  4   10  4   4  4   5   5  5   6   6];
S = [1  1  1  1/2 1/2 1  1/2 1/2 1  2/3 1/3 1/3 2/3 1  1/2 1/2 1  1/2 1/2 1 ...
  1  1/2 1/2 1  1/2 1/2 1  2/3 1/3 1/3 2/3 1  1/2 1/2 1  1/2 1/2 1];
A = full(sparse(I,J,S,26,14));
pa = [0 1 2 3 4 5 6 3 8 9 10 11 12 13 2 15 16 17 18 15 20 21 22 23 24 25];


len = zeros(length(pos),length(pa)-1);
for n = 1:length(pos)
  points = A*pos(n).point;
  for p = 2:size(points,1)
    len(n,p-1) = norm(abs(points(p,1:2)-points(pa(p),1:2)+1));
  end
end

r = zeros(1,length(pa)-1);
for i = 1:length(pa)-1
  ratio = log(len(:,i))-log(len(:,1));
  r(i) = exp(median(ratio));
end

scale = zeros(1,length(pos)); 
boxsize = zeros(1,length(pos));
for n = 1:length(pos)
  ratio = len(n,:)./r;
  boxsize(n) = quantile(ratio,0.85);
  scale(n) = boxsize(n)/(sbin*5);
end

